#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
   TODO Purpose of the file
   @project: HSPyLib
   hspylib.app.firebase.src.main
      @file: __main__.py
   @created: Tue, 4 May 2021
    @author: <B>H</B>ugo <B>S</B>aporetti <B>J</B>unior"
      @site: https://github.com/yorevs/hspylib
   @license: MIT - Please refer to <https://opensource.org/licenses/MIT>

   Copyright 2021, HSPyLib team
"""

import logging as log
import sys
from datetime import datetime

from hspylib.core.tools.commons import get_path, read_version, dirname, syserr
from hspylib.modules.cli.application.application import Application
from hspylib.modules.cli.application.argument_chain import ArgumentChain

from firebase.core.agent_config import AgentConfig
from firebase.core.firebase import Firebase

HERE = get_path(__file__)


class Main(Application):
    """Firebase Agent - Manage your firebase integration"""

    # The application version
    VERSION = read_version(f"{HERE}/.version")

    # Usage message
    USAGE = (HERE / "usage.txt").read_text().format('.'.join(map(str, VERSION)))

    # The welcome message
    WELCOME = (HERE / "welcome.txt").read_text()

    def __init__(self, app_name: str):
        super().__init__(app_name, self.VERSION, self.USAGE, dirname(__file__))
        self.firebase = Firebase()

    def _setup_parameters(self, *params, **kwargs) -> None:
        # @formatter:off
        self._with_option('d', 'dest-dir', True)
        self._with_arguments(
            ArgumentChain.builder()
                .when('operation', 'setup')
                    .end()
                .when('operation', 'upload')
                    .require('db_alias', '.+')
                    .require('files', '.+')
                    .end()
                .when('operation', 'download')
                    .require('db_alias', '.+')
                    .accept('files', '.+')
                    .end()
                .build()
        )
        # @formatter:on

    def _main(self, *params, **kwargs) -> None:
        """Run the application with the command line arguments"""
        log.info(
            self.WELCOME.format(
                self._app_name,
                self.VERSION,
                AgentConfig.INSTANCE.username(),
                AgentConfig.INSTANCE.config_file(),
                datetime.now().strftime("%Y-%m-%d %H:%M:%S"))
        )
        self._exec_application()

    def _exec_application(self) -> None:
        """Execute the specified firebase operation"""
        op = self.getarg('operation')
        if op == 'setup' or not self.firebase.is_configured():
            self.firebase.setup()
        # Already handled above
        if op == 'setup':
            pass
        elif op == 'upload':
            self.firebase.upload(
                self.getarg('db_alias'),
                self.getarg('files').split(',')
            )
        elif op == 'download':
            self.firebase.download(
                self.getarg('db_alias'),
                self.getopt('dest-dir')
            )
        else:
            syserr(f'### Unhandled operation: {op}')
            self.usage(1)


if __name__ == "__main__":
    # Application entry point
    Main('HSPyLib Firebase Agent').INSTANCE.run(sys.argv[1:])



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
   TODO Purpose of the file
   @project: HSPyLib
   hspylib.app.firebase.core
      @file: file_processor.py
   @created: Tue, 4 May 2021
    @author: <B>H</B>ugo <B>S</B>aporetti <B>J</B>unior"
      @site: https://github.com/yorevs/hspylib
   @license: MIT - Please refer to <https://opensource.org/licenses/MIT>

   Copyright 2021, HSPyLib team
"""

import json
import os
from abc import ABC
from typing import List

from hspylib.core.enums.http_code import HttpCode
from hspylib.core.tools.commons import sysout
from hspylib.core.tools.preconditions import check_state, check_not_none, check_argument
from hspylib.modules.fetch.fetch import put, get
from requests.exceptions import HTTPError

from firebase.entity.file_entry import FileEntry


class FileProcessor(ABC):
    """Utility class to upload and download B64 encoded files"""

    @staticmethod
    def upload_files(url: str, file_paths: List[str]) -> int:
        """Upload files to URL"""
        file_data = []
        for f_path in file_paths:
            check_state(os.path.exists(f_path), f'Input file "{f_path}" does not exist')
            file = FileProcessor._read_and_encode(f_path)
            file_data.append(file)
        payload = FileProcessor._to_json(file_data)
        response = put(url, payload)
        check_not_none(response)
        if response.status_code != HttpCode.OK:
            raise HTTPError(
                '{response.status_code} - Unable to upload into={url} with json_string={payload}')
        paths = ', \n\t'.join(file_paths)
        sysout(f"%GREEN%File(s) [\n\t{paths}\n] successfully uploaded to {url}%NC%")

        return len(file_data)

    @staticmethod
    def download_files(url: str, destination_dir: str) -> int:
        """Download files from URL"""
        check_argument(
            destination_dir and os.path.exists(destination_dir),
            "Unable find destination directory: {}", destination_dir)
        response = get(url)
        check_not_none(response)
        check_not_none(response.body)
        if response.status_code != HttpCode.OK:
            raise HTTPError(
                f'{response.status_code} - Unable to download from={url} with response={response}')
        file_data = FileProcessor._from_json(response.body)
        FileProcessor._decode_and_write(destination_dir, file_data)

        return len(file_data)

    @staticmethod
    def _read_and_encode(file_path: str) -> FileEntry:
        """Read and B64 encode a file"""
        return FileEntry(file_path).encode()

    @staticmethod
    def _decode_and_write(destination_dir: str, file_entries: List[dict]) -> None:
        """B64 decode and write entries to file"""
        for entry in file_entries:
            FileEntry.of(
                f"{destination_dir}/{os.path.basename(entry['path'])}",
                entry['data'],
                entry['size']).save()
        files = ', \n\t'.join(list(map(lambda e: e['path'], file_entries)))
        sysout(f'%GREEN%File(s) [\n\t{files}\n] successfully downloaded into {destination_dir}%NC%')

    @staticmethod
    def _to_json(file_data: List[FileEntry]) -> str:
        """Convert the file data into json format"""
        return '[' + ', '.join([str(entry) for entry in file_data]) + ']'

    @staticmethod
    def _from_json(file_data: str) -> List[dict]:
        """Convert json format into file data"""
        return json.loads(file_data)
